; DrumSampler version 1.0
; Written by Jeanette C., find the latest version on:
; http://juliencoder.de
; This software is licensed under the GPL (GNU General Public License)
; version 3 (see COPYING for details).
; Csound version: 6.15 (6.14 should work or GIT version later than January 2020)
; Drum Sampler is designed to sample drum machines or drum kits from other
; synthesizers. It wait for the user to play notes and records, once a note is
; played. It creates round robins automatically, if the same note is played
; more than once.
; For good samples ALWAYS let the sound decay before playing the next note!
; Go to the CsOptions to choose your audio and MIDI i/o.
; Once you've set up everything simply run it with:
; csound autosampler.csd
<CsoundSynthesizer>
<CsOptions>
; Choose your realtime audio module. Remove the semicolon from the beginning
; of the correct line (ONLY from the beginning!!!)
;-+rtaudio=PortAudio ; All platforms
;-+rtaudio=alsa ; Linux: ALSA (Advanced Linux Sound Architecture)
;-+rtaudio=jack ; Linux JACK Audio Connection Kit
;-+rtaudio=mme ; Windows MultiMedia Extension (the Windows standard?)
;-+rtaudio=CoreAudio ; Mac OS standard

; Choose your realtime MIDI module. Rem ove the semicolon from the beginning
; of the correct line
;-+rtmidi=PortMIDI ; All platforms
;-+rtmidi=alsa ; Linux (Advanced Linux Sound Architecture)
;-+rtmidi=cmidi ; Mac OS (CoreMidi?)
;-+rtmidi=mme ; Windows MultiMedia Extension
;-+rtmidi=winmm ; Windows

; Choose your MIDI input port
; ALSA will use the hw:card,subdevice format, you can list your ports
; with amidi -l
; if possible this will also show proper names, especially with USB MIDI
; devices.
; All other MIDI modules only use one number, check your desktop multimedia
; control centre, audio-MIDI setup or whereever you can see your devices
; (maybe inside your DAW, if you use one).
;-M hw:1,0 ; ALSA format)
;-M 1 ; format for all others
; Leave these options and go down to sr and nchnls
-d --messagelevel=2 -o dac -i adc
</CsOptions>
; ==============================================
<CsInstruments>
; sr sets the samplerate, usually either 44100 or 48000Hz.
sr	=	48000
ksmps	=	1
; nchnls tells Csound how many audio channels your soundcard has.
; 2 should work for every soundcard, but you may have more (18 is excessive!)
nchnls	=	18
; Go down to the very bottom to setup the sampler (audio channel(s) and
; the MIDI channel)
0dbfs	=	1
massign 0, 0

; Array for all MIDI notes. Count how often a note has been played
gkNotes[] init 128

; Shut down Csound when the work is done
instr Exit
	exitnow
endin

; Record a note
; p4 = iNote, p5 = iRobin
instr Record
	iNote init p4
	iRobin init p5
	Sname = chnget:S("DrumSampler.Name")
	iStereo = chnget:i("DrumSampler.Stereo")

	; Initialise internal controls
	kDone init 0 ; 1 when finished
	kNow timeinsts

	; Create audio signals to be saved and RMS control outputs
	aLeft = chnget:a("DrumSampler.Left")
	aRight = chnget:a("DrumSampler.Right")
	kRms1 init 1
	kRms2 init 0

	Sfilename = sprintf("Samples/%s-%03d-%d.wav", Sname, iNote, iRobin)

	if (k(iStereo) == 0) then
		kRms1 = rms(aLeft)
		fout(Sfilename, -1, aLeft)
	else
		kRms1 = rms(aLeft)
		kRms2 = rms(aRight)
		fout(Sfilename, -1, aLeft, aRight)
	endif

	; Wait for note to decay
	if ((kRms1 <.0001) && (kRms2 <.0001) && (kNow >.05)) then
		kDone = 1
		chnset(k(1), "DrumSampler.Ready")
		printks("\nNote %d round robin %d has decayed.", 1/kr, iNote, (gkNotes[iNote] -1))
	endif ; rms below audible threshold

	; Quit when Done
	if (kDone == 1) then
		turnoff
	endif
endin

; Write one SFZ file
; p4 = iNote, p5 = iRonbins
instr WriteSFZ
	; Initialise parameters
	iNote init p4
	iRobins init p5
	iRobinStep init 1 / iRobins
	Sname = chnget:S("DrumSampler.Name")
	iHat1 = chnget:i("DrumSampler.Hat1")
	iHat2 = chnget:i("DrumSampler.Hat2")
	iHat3 = chnget:i("DrumSampler.Hat3")

	; Create filename, a special filename for hats
	if ((iNote == iHat1) || (iNote == iHat2) || (iNote == iHat3)) then
		Sfilename = sprintf("%s-hats.sfz", Sname)
		iIsHat init 1
	else ; a normal note based filename
		Sfilename = sprintf("%s-%03d.sfz", Sname, iNote)
		iIsHat init 0
	endif

	; Start printing information
	fprints(Sfilename, "<group> amp_veltrack=98 key=%d loop_mode=one_shot lovel=1 hivel=127", iNote)
	if (iIsHat == 1) then
		fprints(Sfilename, " group=1 off_by=1")
	endif

	; Go through the notes
	iIndex init 0
	while (iIndex <iRobins) do
		iLorand = iIndex * iRobinStep
		iHirand = (iIndex + 1) * iRobinStep
		fprints(Sfilename, "\n<region> sample=Samples\\\\%s-%03d-%d.wav lorand=%f hirand=%f", Sname, iNote, iIndex, iLorand, iHirand)
		iIndex += 1
	od
	fprints(Sfilename, "\n")

	; The work is done, turnoff
	chnset(k(1), "DrumSampler.Ready")
	turnoff
endin

; Manage writing of all SFZ files
instr MakeSFZ
	; Initialise parameters
	kIndex init 0 ; note index into global gkNotes array
	kDone init 0 ; set to 1 when finished
	chnset(1, "DrumSampler.Ready")
	
	; Go through all notes
	if (chnget:k("DrumSampler.Ready") == 1) then
		if (gkNotes[kIndex] >0) then
			chnset(k(0), "DrumSampler.Ready")
			schedulek("WriteSFZ", 0, -1, kIndex, gkNotes[kIndex])
		endif

		; Increase note or quit when finished
		if (kIndex <127) then
			kIndex += 1
		else
			kDone = 1
		endif
	endif

	if (kDone == 1) then
		schedulek("Exit", 0, .1)
	endif
endin

; The drumsampler main instrument
; p4 = Sname, p5 = iLeft, p6 = iRight, p7 = iMidiChannel, p8 = iHat11, 
; p9 = iHat2, p10 = iHat3 (note numbers of hihats for mute group, set any
; or all to -1 if not appropriate)
instr DrumSampler
	; Initialise parameters
	Sname = strget(p4)
	iLeft init p5
	iRight init p6
	iMidiChannel init p7
	iHat1 init p8
	iHat2 init p9
	iHat3 init p10

	; Initialise internal controls
	kPaused init 1 ; one when not recording, i.e. sound exploring
	kDone init 0 ; 1 when finished
	; midiin control outputs
	kStatus init 0 
	kChannel init 0
	kData1 init 0
	kData2 init 0
	; Keyboard navigation
	kKey init 0 ; computer keyboard key for commands
	kPressed init 0 ; second output of sensekey for keyboard commands

	; Create Samples folder
	iMkdir = system_i(1, "mkdir Samples")

	; Create channels to propagate global data
	chn_k("DrumSampler.Ready", 3) ; 1 when ready to record
	chnset(1, "DrumSampler.Ready")
	chn_S("DrumSampler.Name", 3)
	chnset(Sname, "DrumSampler.Name")
	if (iRight == -1) then
		iStereo init 0
	else
		iStereo init 1
	endif
	chn_k("DrumSampler.Stereo", 3)
	chnset(iStereo, "DrumSampler.Stereo")
	chn_k("DrumSampler.Hat1", 3)
	chnset(iHat1, "DrumSampler.Hat1")
	chn_k("DrumSampler.Hat2", 3)
	chnset(iHat2, "DrumSampler.Hat2")
	chn_k("DrumSampler.Hat3", 3)
	chnset(iHat3, "DrumSampler.Hat3")
	chn_a("DrumSampler.Left", 3)
	chn_a("DrumSampler.Right", 3)

	; Get the audio
	aLeft init 0
	aRight init 0

	if (k(iStereo) == 0) then
		aLeft inch iLeft
	else
		aLeft, aRight inch iLeft, iRight
	endif

	; Copy audio to channels
	chnset(aLeft, "DrumSampler.Left")
	chnset(aRight, "DrumSampler.Right")

	; Output instructions
	prints("\n\n----------------------------------------")
	prints("\nDrum Sampler started, press 'r' to record or 'q' to quit.")
	prints("\nCurrently in play/exploration mode, twiddle your knobs, find your sound.")

	; Process keyboard input
	kKey, kPressed sensekey
	if (kPressed == 1) then ; a key is pressed
		if ((kKey == 80) || (kKey == 112)) then ; p for pause/play
			kPaused = 1
			chnset(k(0), "DrumSampler.Ready")
			printks("\nYou're in play/exploration mode, press 'r' to record again.", 1/kr)
		elseif ((kKey == 82) || (kKey == 114)) then ; r for record
			kPaused = 0
			chnset(k(1), "DrumSampler.Ready")
			printks("\nYou're in record mode.", 1/kr)
		elseif ((kKey == 81) || (kKey == 113)) then ; q for quit
			kDone = 1
			schedulek("MakeSFZ", 0, -1)
			printks("\nExiting drum sampler.", 1/kr)
		endif ; key switching
	endif ; key pressed

	; Main commands/record
	kStatus, kChannel, kData1, kData2 midiin
	if (kPaused == 0) then ; record a note
		if (chnget:k("DrumSampler.Ready") == 1) then ; we can record
			if ((kStatus == 144) && (kChannel == iMidiChannel)) then
				chnset(k(0), "DrumSampler.Ready")
				schedulek("Record", 0, -1, kData1, gkNotes[kData1])
				gkNotes[kData1] = gkNotes[kData1] + 1
			endif ; a note-on on our channel
		endif ; ready
	endif ; !paused

	; If done, quit this instrument
	if (kDone == 1) then
		turnoff
	endif

	; Output audio
	if (iStereo == 0) then
		outs(aLeft, aLeft)
	else
		outs(aLeft, aRight)
	endif
endin

</CsInstruments>
; ==============================================
<CsScore>
; Set up the drum sampler
; p4 = Sname, p5 = iLeftChannel, p6 = iRightChannel(-1 for mono),
; p7 = iMidiChannel(1-16)
; The example below creates samples with the name RD8-note_number, recorded
; mono on audio channel 6 (right channel is -1) and on MIDI channel 1.
; There are two hihats on MIDI note 42 and 46 and no third hihat (-1).
;i"DrumSampler" 0 z "RD8" 6 -1 1 42 46 -1
e
</CsScore>
</CsoundSynthesizer>
